import os
import sys
import time
from StringIO import StringIO

sys.path.append("../../../../ecamber/src/")
from soft.utils.camber_params_utils import *
from soft.utils.camber_utils import *
from soft.utils.camber_io_utils import *
from soft.structs.multigene import *
from alnsoft.structs.aln_filenames import *
from alnsoft.utils.camber_aln_utils import *
from alnsoft.utils.camber_phylo_utils import *


parameters = overwriteParameters(sys.argv)
parameters = readParameters()
parameters = readParametersFromFile("../../../config/config_aln_paths.txt", parameters)
parameters = readParametersFromFile("../../../config/config_aln_params.txt", parameters)

phase = 0
if "PHASE" in parameters:
    phase = int(parameters["PHASE"])

ensure_dir(parameters["MUTATIONS_RESULTS"])

diffTime("params")
strains = readStrainsInfo()
diffTime("info")
print((strains.allStrains()))

diffTime("sequences")
#readExtAnnotations(strains.allStrains())
diffTime("annotations")
multigene_graph = MultigeneGraph()
diffTime()
#input_fn = parameters_map["CLOSURE_RESULTS_GRAPHS"] + "mult_details.txt"
input_fn = parameters_map["CLOSURE_RESULTS_GRAPHS"] + "cluster-comb-det-" + str(phase) + ".txt"
input_fh = open(input_fn)
multigene_graph = readMultigenesGraphFast2(strains, multigene_graph, input_fh)
input_fh.close()
diffTime("graph")


for genome_id in strains.allGenomes():
    print((len(multigene_graph.nodes_by_strain[genome_id])))
diffTime("strain")

#input_fn2 = parameters_map["CLOSURE_RESULTS_GRAPHS"] + "cluster-det2-" + str(phase) + ".txt"
input_fh2 = open(input_fn)
clusters = readClusters2(multigene_graph, input_fh=input_fh2)
clusters.classifyClusters()
input_fh2.close()
diffTime("classified")

strains_ord = strains.getOrderedGenomes(strains.allGenomes())

cluster_fh = open(input_fn)
clusters, id_cluster_map = readClusters(cluster_fh, strains)
cluster_fh.close()

anns_fn = parameters_map["CLOSURE_RESULTS"] + "names_anns.txt"
if os.path.exists(anns_fn):
    anns_fh = open(anns_fn)
    names_mapping = readNamesMapping(anns_fh)
    anns_fh.close()
else:
    names_mapping = None

#aln_type = parameters["ALN_TYPE"] 
aln_type = "DNA"

aln_filenames = AlnFilenames()
filenames_fn = parameters["PHYLOGENY_FOLDER"] + "filenames.txt"
filenames_fh = open(filenames_fn)

lines = filenames_fh.readlines()
for line in lines:
    aln_filenames.addMappingByLine(line)

output_fn = parameters["MUTATIONS_RESULTS"] + "gap-mutations-" + str(phase) + ".txt";
output_fh = open(output_fn, "w")
writeMutationsHeader(output_fh, strains);

for cluster in list(clusters.anchors.values()):
    if len(cluster.nodes) == 1:
        print("size-1 cluster", cluster.name) 
        continue
    mix_cluster_id = clusterLongestMixID(cluster)
    if not aln_filenames.hasClusterID(mix_cluster_id):
        aln_filenames.addMapping(mix_cluster_id)
    filename_id = aln_filenames.getMapping(mix_cluster_id)
    if aln_type == "AA":
        aln_fn = parameters["ALN_PHYLOGENY_FOLDER_AA"] + filename_id + ".aln"
    else:
        aln_fn = parameters["ALN_PHYLOGENY_FOLDER"] + filename_id + ".aln"
    if not os.path.exists(aln_fn):
        print("NO FILE: " + aln_fn + ";" + str(len(cluster.nodes)))
    if not os.path.exists(aln_fn):
        continue
    align_fh = open(aln_fn);

    aligns_seq = readSequencesFromAlignmentFile(align_fh);
    gene_gaps = gapsList(aligns_seq, cluster.name)

    for gene_gap in gene_gaps:
        
        text = ""
        text += cluster.name + "\t"    
        text += filename_id + "\t"    
        gene_id = cluster.name
            
        if not clusters == None:
            mapped_gene_id = mapID(gene_id, clusters, id_cluster_map, strains_ord)
            if mapped_gene_id == "x":
                mapped_gene_id = gene_id
        else:
            mapped_gene_id = gene_id
    
        gene_name = ""
        if names_mapping != None:
            gene_name = mapGeneName(gene_id, clusters, id_cluster_map, strains_ord, names_mapping)
            if gene_name == "x":
                gene_name = mapped_gene_id
    
        #gene_name = cluster.geneName(genomes_list) 
        text += gene_name + "\t"         
        text += "indel" + "-" + str(gene_gap.len) + "\t"
            
        text += str(gene_gap.start) + ";" + str(gene_gap.end) + "\t"
        
        for strain_id in strains.allStrains():
            strain = strains.strain(strain_id)
            if len(cluster.nodesByStrain(strain_id)) == 0:
                plas_id = ""
                for plasmid_id in strain.plasmids:
                    if len(cluster.nodesByStrain(plasmid_id)) > 0:
                        plas_id = plasmid_id
                if plas_id == "":
                    text += "?"
                else:
                    if plas_id in gene_gap.genomes:
                        text += "D"#+str(gene_gap.strains[plas_id])
                    else:
                        text += "I"
                    
            else:
                if strain_id in gene_gap.genomes:
                    text += "D"#+str(gene_gap.strains[strain_id])
                else:
                    text += "I"
            text += "\t"              
  #      if gene_gap.support(strains.scStrains())>=3 and gene_gap.supportNeg(strains.resStrains())>=3:
        output_fh.write(text + "\n")
        print(filename_id)
        print(text)

output_fh.close()
